/*
 *	Wireless Tools
 *
 * Tool for querying the range parameters of the wireless driver
 * You need to link this code against "iwlib.c" and "-lm".
 *
 * This file is released under the GPL license.
 *     Copyright (c) 2009 Reto Gantenbein <reto.gantenbein@iam.unibe.ch>
 */

#include "iwlib.h"		/* Header */


/************************* DISPLAY ROUTINES **************************/
/*------------------------------------------------------------------*/
/*
 * Print on the screen in a neat fashion all the info we have collected
 * on a device.
 */
static void
display_range(iwrange *	range,
	      char *	ifname)
{
  int	i;

  printf("%-8.16s  ", ifname);
  printf("Driver parameter ranges");
  printf("\n          ");

  /* WE version */
  printf("- Wireless Extension version: ");
  printf("%i (compiled), %i (source)", range->we_version_compiled,
	range->we_version_source);
  printf("\n          ");

  /* Bitrates */
  printf("- Bitrate Capabilities: ");
  if(range->bitrate_capa == 0)
    printf("NA");
  else
    printf("%#.4x", range->bitrate_capa);
  printf("\n          ");
  printf("  Bitrates: ");
  if(range->num_bitrates == 0)
    printf("NA");
  else
    {
      for(i=0; i<range->num_bitrates; i++)
	{
	  if((i > 0) && (i%10 == 0))
	    printf("\n                      ");
	  printf("%.1f ", range->bitrate[i] / MEGA);
	}
      printf("Mbit/s  ");
    }
  printf("\n          ");

  /* Througput */
  printf("- Maximal throughput: ");
  if(range->throughput == 0)
    printf("NA");
  else
    printf("%.2f Mbit/s  ", range->throughput / MEGA);
  printf("\n          ");

  /* Frequency */
  printf("- Number of channels: ");
  if(range->num_channels == 0)
    printf("NA");
  else
    printf("%i", range->num_channels);
  printf("\n          ");
  printf("  Frequencies: ");
  if(range->num_frequency == 0)
    printf("n.a.");
  else
    {
      for(i=0; i<range->num_frequency; i++)
	{
	if((i > 0) && (i%7 == 0))
	    printf("\n                         ");
	  printf("%.3f ", iw_freq2float(&(range->freq[i])) / GIGA);
	}
      printf("GHz  ");
    }
  printf("\n          ");

  /* Network ID */
  printf("- NWID: ");
  if(range->max_nwid == 0)
    printf("NA");
  else
    printf("min %i, max %i  ", range->min_nwid, range->max_nwid);
  printf("\n          ");

  /* Sensitivity */
  printf("- Sensitivity: ");
  if(range->sensitivity == 0)
    printf("NA");
  else
    {
      if(range->sensitivity < 0)
	printf("%d dBm", range->sensitivity);
      else
	printf("%d", range->sensitivity);
    }
  printf("\n          ");

  /* RTS threshold */
  printf("- RTS threshold: ");
  if(range->max_rts == 0)
    printf("NA");
  else
    printf("min %i bytes, max %i bytes", range->min_rts, range->max_rts);
  printf("\n          ");

  /* Fragmentation threshold */
  printf("- Fragmentation threshold: ");
  if(range->max_frag == 0)
    printf("NA");
  else
    printf("min %i bytes, max %i bytes", range->min_frag, range->max_frag);
  printf("\n          ");

  /* Txpower */
  printf("- TX Power capabilities: ");
  if(range->txpower_capa == 0)
    printf("NA");
  else
    printf("%#.4x  ", range->txpower_capa);
  printf("\n          ");
  printf("  TX Power states: ");
  if(range->num_txpower == 0)
    printf("NA");
  else
    {
      for(i=0; i<range->num_txpower; i++)
	printf("%i ", range->txpower[i]);
    }
  printf("\n          ");

  /* Retry */
  printf("- Retry capabilities: ");
  if(range->retry_capa == 0)
    printf("NA");
  else
    printf("%#.4x", range->retry_capa);
  printf("\n          ");
  printf("  Retry flags: ");
  if(range->retry_flags == 0)
    printf("NA");
  else
    printf("%#.4x", range->retry_flags);
  printf("\n          ");
  printf("  Number of retries: ");
  if(range->max_retry == 0)
    printf("NA");
  else
    printf("min %i, max %i", range->min_retry, range->max_retry);
  printf("\n          ");
  printf("  Retry lifetime flags: ");
  if(range->r_time_flags == 0)
    printf("NA");
  else
    printf("%#.4x", range->r_time_flags);
  printf("\n          ");
  printf("  Retry lifetime: ");
  if(range->max_r_time == 0)
    printf("NA");
  else
    printf("min %i, max %i", range->min_r_time, range->max_r_time);
  printf("\n          ");

  /* Power Management */
  printf("- Power Management capabilities: ");
  if(range->pm_capa == 0)
    printf("NA");
  else
    printf("%#.4x", range->pm_capa);
  printf("\n          ");
  printf("  Power Management period flags: ");
  if(range->pmp_flags == 0)
    printf("NA");
  else
    printf("%#.4x", range->pmp_flags);
  printf("\n          ");
  printf("  Power Management period: ");
  if(range->max_pmp == 0)
    printf("NA");
  else
    printf("  min %i ms, max %i ms", range->min_pmp/1000, range->max_pmp/1000);
  printf("\n          ");
  printf("  Power Management timeout flags: ");
  if(range->pmt_flags == 0)
    printf("NA");
  else
    printf("%#.4x", range->pmt_flags);
  printf("\n          ");
  printf("  Power Management timeout: ");
  if(range->max_pmt == 0)
    printf("NA");
  else
    printf("min %i ms, max %i ms", range->min_pmt/1000, range->max_pmt/1000);
  printf("\n          ");
  printf("  Power Management saving flags: ");
  if(range->pms_flags == 0)
    printf("NA");
  else
    printf("%#.4x", range->pms_flags);
  printf("\n          ");
  printf("  Power Management saving: ");
  if(range->max_pms == 0)
    printf("NA");
  else
    printf("min %i s, max %i s", range->min_pms/1000, range->max_pms/1000);
  printf("\n          ");
  printf("- Driver modulation capabilities: ");
  if(range->modul_capa == 0)
    printf("NA");
  else
    printf("%#.4x", range->modul_capa);
  printf("\n          ");

  /* Link quality */
  printf("- Absolut link quality: ");
  printf("%d, level %d, noise %d, updated %d", range->max_qual.qual,
	range->max_qual.level, range->max_qual.noise, range->max_qual.updated);
  printf("\n          ");
  printf("  Average link quality: ");
  printf("%d, level %d, noise %d, updated %d", range->avg_qual.qual,
	range->avg_qual.level, range->avg_qual.noise, range->avg_qual.updated);
  printf("\n          ");

  /* Encoding */
  printf("- Encoding capabilities: ");
  if(range->enc_capa == 0)
    printf("NA");
  else
    printf("%#.4x", range->enc_capa);
  printf("\n          ");
  printf("  Encoding sizes: ");
  if(range->num_encoding_sizes == 0)
    printf("NA");
  else
    {
      for(i=0; i<range->num_encoding_sizes; i++)
	printf("%i ", range->encoding_size[i]);
    }
  printf("\n          ");
  printf("  Maximal encoding tokens: ");
  if(range->max_encoding_tokens == 0)
    printf("NA");
  else
    printf("%i ", range->max_encoding_tokens);
  printf("\n          ");
  printf("  Encoding login index: %i", range->max_encoding_tokens);
  printf("\n          ");

  /* Event capabilities */
  printf("- Event Capabilities: ");
  for (i = 0; i < 6; i++)
    {
      if(range->event_capa[i] != 0)
	{
	  printf("%#.4x", range->event_capa[i]);
	  if(i < 5)
	    printf("\n                                ");
	}
    }
  printf("\n");
}

/*------------------------------------------------------------------*/
/*
 * Print range
 */
static int
print_range(int	 	skfd,
	    char *	ifname,
	    char *	args[],
	    int		count)
{
  int		rc;
  iwrange	range;

  /* Avoid "Unused parameter" warning */
  args = args; count = count;

  rc = iw_get_range_info(skfd, ifname, &range);
  switch(rc)
    {
    case 0:	/* Success */
      /* Display it ! */
      display_range(&range, ifname);
      break;

    case -ENOTSUP:
      fprintf(stderr, "%-8.16s  no wireless extensions.\n\n", ifname);
      break;

    default:
      fprintf(stderr, "%-8.16s  %s\n\n", ifname, strerror(-rc));
    }
  return(rc);
}

/*------------------------------------------------------------------*/
/*
 * Display help
 */
static inline void
iw_usage(void)
{
  fprintf(stderr,   "Usage: iwrange [interface]\n");
}

/******************************* MAIN ********************************/

/*------------------------------------------------------------------*/
/*
 * The main !
 */
int
main(int	argc,
     char **	argv)
{
  int skfd;		/* generic raw socket desc.	*/
  int goterr = 0;

  /* Create a channel to the NET kernel. */
  if((skfd = iw_sockets_open()) < 0)
    {
      perror("socket");
      exit(-1);
    }

  /* No argument : show the list of all device + info */
  if(argc == 1)
    iw_enum_devices(skfd, &print_range, NULL, 0);
  else
    /* Special case for help... */
    if((!strcmp(argv[1], "-h")) || (!strcmp(argv[1], "--help")))
      iw_usage();

    else
      /* The device name must be the first argument */
      if(argc == 2)
	goterr = print_range(skfd, argv[1], NULL, 0);

  /* Close the socket. */
  iw_sockets_close(skfd);

  return(goterr);
}
