/*
 * wlanmodel/cUDPSocketRTScheduler.h
 *
 * Copyright (C) 2008-2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __CUDPSOCKETRTSCHEDULER_H__
#define __CUDPSOCKETRTSCHEDULER_H__

#include <cerrno>
#include <omnetpp.h>
#include <platdep/timeutil.h>
#include <platdep/sockets.h>

/** Scheduler class for message handling in the OMNeT++ simulation */
class cUDPSocketRTScheduler : public cScheduler
{
  protected:
    // config
    int port;
    cModule *module;
    cMessage *notificationMsg;
    char *recvBuffer;
    int recvBufferSize;
    int *numBytesPtr;

    // state
    timeval baseTime;
    SOCKET listenerSocket;
    SOCKET connSocket;

    virtual void setupListener();
    virtual bool receiveWithTimeout(long usec);
    virtual int receiveUntil(const timeval& targetTime);

  public:
    /** Constructor */
    cUDPSocketRTScheduler();

    /** Destructor */
    virtual ~cUDPSocketRTScheduler();

    /** Called at the beginning of a simulation run */
    virtual void startRun();

    /** Called at the end of a simulation run */
    virtual void endRun();

    /** Recalculates "base time" from current wall clock time */
    virtual void executionResumed();

    /** To be called from the module which wishes to receive data from the
     * socket. The method must be called from the module's initialize()
     * function */
    virtual void setInterfaceModule(cModule *module,
									cMessage *notificationMsg,
									char *recvBuffer,
									int recvBufferSize,
									int *numBytesPtr);

    /** Scheduler function -- it comes from cScheduler interface */
    virtual cMessage *getNextEvent();

    /** Send to message to node */
    int sendMessage(const char *msg,
					const size_t numBytes,
					const sockaddr *nodeAddr,
					const socklen_t addrSize);
};

#endif
